/hg/icedtea-web: Massively improved offline abilities. Added Xof...

jvanek at icedtea.classpath.org jvanek at icedtea.classpath.org
Tue Aug 5 15:29:01 UTC 2014


changeset 178f758bd94d in /hg/icedtea-web
details: http://icedtea.classpath.org/hg/icedtea-web?cmd=changeset;node=178f758bd94d
author: Jiri Vanek <jvanek at redhat.com>
date: Tue Aug 05 17:28:31 2014 +0200

	Massively improved offline abilities. Added Xoffline switch to force work without inet connection.


diffstat:

 ChangeLog                                               |  25 ++++
 NEWS                                                    |   1 +
 netx/net/sourceforge/jnlp/JNLPFile.java                 |   6 +-
 netx/net/sourceforge/jnlp/Launcher.java                 |  28 ++--
 netx/net/sourceforge/jnlp/cache/CacheUtil.java          |   7 -
 netx/net/sourceforge/jnlp/cache/ResourceTracker.java    |  93 +++++++---------
 netx/net/sourceforge/jnlp/resources/Messages.properties |   1 +
 netx/net/sourceforge/jnlp/runtime/Boot.java             |   2 +
 netx/net/sourceforge/jnlp/runtime/JNLPRuntime.java      |  57 ++++++++++
 netx/net/sourceforge/jnlp/services/XBasicService.java   |  14 +--
 netx/net/sourceforge/jnlp/util/XDesktopEntry.java       |   4 +-
 11 files changed, 148 insertions(+), 90 deletions(-)

diffs (410 lines):

diff -r c6e06c9b4bfb -r 178f758bd94d ChangeLog
--- a/ChangeLog	Tue Aug 05 11:13:20 2014 -0400
+++ b/ChangeLog	Tue Aug 05 17:28:31 2014 +0200
@@ -1,3 +1,28 @@
+2014-08-05  Jiri Vanek  <jvanek at redhat.com>
+	Massively improved offline abilities. Added Xoffline switch to force work without inet connection.
+	* NEWS: updated
+	* netx/net/sourceforge/jnlp/JNLPFile.java: (openURL) is now using properly 
+	cached file instead of direct online one.
+	* netx/net/sourceforge/jnlp/Launcher.java: launcher now can run offline-only
+	jnlp files if Xoffline specified.
+	* netx/net/sourceforge/jnlp/cache/CacheUtil.java: removed suspicious removal
+	of http/https dirs in cache
+	* netx/net/sourceforge/jnlp/cache/ResourceTracker.java: misleading 
+	(getInputStream) method removed (initializeResource) check for connection
+	before downlaodin (unless Xforceoffline specified). If environment is offline
+	it do not attempt any url connections or writing to cache
+	* netx/net/sourceforge/jnlp/resources/Messages.properties: added (BXoffline)
+	description
+	* netx/net/sourceforge/jnlp/runtime/Boot.java: added help and read for Xoffline
+	* netx/net/sourceforge/jnlp/runtime/JNLPRuntime.java: added flags of (offlineForced)
+	and (onlineDetected) with getters and setters. Added utility method (detectOnline)
+	to recognize whether environment is onliune by resovling inet addres of	host
+	of not file url.
+	* netx/net/sourceforge/jnlp/services/XBasicService.java: (isOffline) now uses 
+	above implemented mechanism.
+	* netx/net/sourceforge/jnlp/util/XDesktopEntry.java: now writes real url into
+	desktop icon
+
 2014-08-05  Andrew Azores  <aazores at redhat.com>
             Lukasz Dracz  <ldracz at redhat.com>
             
diff -r c6e06c9b4bfb -r 178f758bd94d NEWS
--- a/NEWS	Tue Aug 05 11:13:20 2014 -0400
+++ b/NEWS	Tue Aug 05 17:28:31 2014 +0200
@@ -9,6 +9,7 @@
 CVE-XXXX-YYYY: http://www.cve.mitre.org/cgi-bin/cvename.cgi?name=XXXX-YYYY
 
 New in release 1.6 (2014-XX-XX):
+* Massively improved offline abilities. Added Xoffline switch to force work without inet connection.
 * Improved to be able to run with any JDK
 * JDK 6 and older no longer supported
 * JDK 8 support added (URLPermission granted if applicable)
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/JNLPFile.java
--- a/netx/net/sourceforge/jnlp/JNLPFile.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/JNLPFile.java	Tue Aug 05 17:28:31 2014 +0200
@@ -16,6 +16,8 @@
 
 package net.sourceforge.jnlp;
 
+import java.io.File;
+import java.io.FileInputStream;
 import static net.sourceforge.jnlp.runtime.Translator.R;
 
 import java.io.IOException;
@@ -299,8 +301,8 @@
         try {
             ResourceTracker tracker = new ResourceTracker(false); // no prefetch
             tracker.addResource(location, version, null, policy);
-
-            return tracker.getInputStream(location);
+            File f = tracker.getCacheFile(location);
+            return new FileInputStream(f);
         } catch (Exception ex) {
             throw new IOException(ex.getMessage());
         }
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/cache/CacheUtil.java
--- a/netx/net/sourceforge/jnlp/cache/CacheUtil.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/cache/CacheUtil.java	Tue Aug 05 17:28:31 2014 +0200
@@ -602,13 +602,6 @@
             }
             lruHandler.store();
 
-            /*
-             * FIXME: if cacheDir is for example $USER_HOME and they have a folder called http
-             * and/or https. These would get removed.
-             */
-            remove.add(cacheDir + File.separator + "http");
-            remove.add(cacheDir + File.separator + "https");
-
             removeSetOfDirectories(remove);
 
         }
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/cache/ResourceTracker.java
--- a/netx/net/sourceforge/jnlp/cache/ResourceTracker.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/cache/ResourceTracker.java	Tue Aug 05 17:28:31 2014 +0200
@@ -27,9 +27,11 @@
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.HttpURLConnection;
+import java.net.InetAddress;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
+import java.net.UnknownHostException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.ArrayList;
@@ -408,36 +410,6 @@
     }
 
     /**
-     * Returns an input stream that reads the contents of the
-     * resource.  For non-cacheable resources, an InputStream that
-     * reads from the source location is returned.  Otherwise the
-     * InputStream reads the cached resource.
-     * <p>
-     * This method will block while the resource is downloaded to
-     * the cache.
-     * </p>
-     *
-     * @param location location of resource which stream will be obtained
-     * @return the stream of the resource
-     * @throws IOException if there was an error opening the stream
-     * @throws IllegalResourceDescriptorException if the resource is not being tracked
-     */
-    public InputStream getInputStream(URL location) throws IOException {
-        try {
-            Resource resource = getResource(location);
-            if (!(resource.isSet(DOWNLOADED) || resource.isSet(ERROR)))
-                waitForResource(location, 0);
-
-            if (resource.getLocalFile() != null)
-                return new FileInputStream(resource.getLocalFile());
-
-            return resource.getLocation().openStream();
-        } catch (InterruptedException ex) {
-            throw new IOException("wait was interrupted");
-        }
-    }
-
-    /**
      * Wait for a group of resources to be downloaded and made
      * available locally.
      *
@@ -804,6 +776,10 @@
      * @param resource the resource to initialize
      */
     private void initializeResource(Resource resource) {
+        //verify connection
+        if(!JNLPRuntime.isOfflineForced()){
+            JNLPRuntime.detectOnline(resource.getLocation()/*or doenloadLocation*/);
+        }
         resource.fireDownloadEvent(); // fire CONNECTING
 
         CacheEntry entry = new CacheEntry(resource.getLocation(), resource.getRequestVersion());
@@ -811,34 +787,43 @@
 
         try {
             File localFile = CacheUtil.getCacheFile(resource.getLocation(), resource.getDownloadVersion());
+            long size = 0;
+            boolean current = true;
+            //this can be null, as it is always filled in online mode, and never read in offline mode
+            URLConnection connection = null;
+            if (localFile != null) {
+                size = localFile.length();
+            } else if (!JNLPRuntime.isOnline()) {
+                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "You are trying to get resource " + resource.getLocation().toExternalForm() + " but you are in offline mode, and it is not in cache. Attempting to continue, but you may expect failure");
+            }
+            if (JNLPRuntime.isOnline()) {
+                // connect
+                URL finalLocation = findBestUrl(resource);
 
-            // connect
-            URL finalLocation = findBestUrl(resource);
+                if (finalLocation == null) {
+                    OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Attempted to download " + resource.getLocation() + ", but failed to connect!");
+                    throw new NullPointerException("finalLocation == null"); // Caught below
+                }
 
-            if (finalLocation == null) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Attempted to download " + resource.getLocation() + ", but failed to connect!");
-                throw new NullPointerException("finalLocation == null"); // Caught below
-            }
+                resource.setDownloadLocation(finalLocation);
+                connection = finalLocation.openConnection(); // this won't change so should be okay unsynchronized
+                connection.addRequestProperty("Accept-Encoding", "pack200-gzip, gzip");
 
-            resource.setDownloadLocation(finalLocation);
-            URLConnection connection = finalLocation.openConnection(); // this won't change so should be okay unsynchronized
-            connection.addRequestProperty("Accept-Encoding", "pack200-gzip, gzip");
-
-            int size = connection.getContentLength();
-            boolean current = CacheUtil.isCurrent(resource.getLocation(), resource.getRequestVersion(), connection.getLastModified()) && resource.getUpdatePolicy() != UpdatePolicy.FORCE;
-            if (!current) {
-                if (entry.isCached()) {
-                    entry.markForDelete();
-                    entry.store();
-                    // Old entry will still exist. (but removed at cleanup)
-                    localFile = CacheUtil.makeNewCacheFile(resource.getLocation(), resource.getDownloadVersion());
-                    CacheEntry newEntry = new CacheEntry(resource.getLocation(), resource.getRequestVersion());
-                    newEntry.lock();
-                    entry.unlock();
-                    entry = newEntry;
+                size = connection.getContentLength();
+                current = CacheUtil.isCurrent(resource.getLocation(), resource.getRequestVersion(), connection.getLastModified()) && resource.getUpdatePolicy() != UpdatePolicy.FORCE;
+                if (!current) {
+                    if (entry.isCached()) {
+                        entry.markForDelete();
+                        entry.store();
+                        // Old entry will still exist. (but removed at cleanup)
+                        localFile = CacheUtil.makeNewCacheFile(resource.getLocation(), resource.getDownloadVersion());
+                        CacheEntry newEntry = new CacheEntry(resource.getLocation(), resource.getRequestVersion());
+                        newEntry.lock();
+                        entry.unlock();
+                        entry = newEntry;
+                    }
                 }
             }
-
             synchronized (resource) {
                 resource.setLocalFile(localFile);
                 // resource.connection = connection;
@@ -851,7 +836,7 @@
             }
 
             // update cache entry
-            if (!current) {
+            if (!current && JNLPRuntime.isOnline()) {
                 entry.setRemoteContentLength(connection.getContentLengthLong());
                 entry.setLastModified(connection.getLastModified());
             }
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/resources/Messages.properties
--- a/netx/net/sourceforge/jnlp/resources/Messages.properties	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/resources/Messages.properties	Tue Aug 05 17:28:31 2014 +0200
@@ -257,6 +257,7 @@
 BXnofork    = Do not create another JVM.
 BXclearcache= Clean the JNLP application cache.
 BXignoreheaders= Skip jar header verification.
+BXoffline   = Prevent ITW network connection. Only cache will be used. Application can still connect.
 BOHelp      = Print this message and exit.
 
 # Cache
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/runtime/Boot.java
--- a/netx/net/sourceforge/jnlp/runtime/Boot.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/runtime/Boot.java	Tue Aug 05 17:28:31 2014 +0200
@@ -120,6 +120,7 @@
             + "  -Xnofork              " + R("BXnofork") + "\n"
             + "  -Xclearcache          " + R("BXclearcache") + "\n"
             + "  -Xignoreheaders       " + R("BXignoreheaders") + "\n"
+            + "  -Xoffline             " + R("BXoffline") + "\n"
             + "  -help                 " + R("BOHelp") + "\n";
 
     private static final String doubleArgs = "-basedir -jnlp -arg -param -property -update";
@@ -219,6 +220,7 @@
      */
     public Void run() {
         JNLPRuntime.setSecurityEnabled(null == getOption("-nosecurity"));
+        JNLPRuntime.setOfflineForced(null != getOption("-Xoffline"));
         JNLPRuntime.initialize(true);
 
         /*
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/runtime/JNLPRuntime.java
--- a/netx/net/sourceforge/jnlp/runtime/JNLPRuntime.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/runtime/JNLPRuntime.java	Tue Aug 05 17:28:31 2014 +0200
@@ -24,7 +24,10 @@
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.net.Authenticator;
+import java.net.InetAddress;
 import java.net.ProxySelector;
+import java.net.URL;
+import java.net.UnknownHostException;
 import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
 import java.security.AllPermission;
@@ -159,6 +162,12 @@
     
     /** allows 301.302.303.307.308 redirects to be followed when downloading resources*/
     private static boolean allowRedirect = false;;
+    
+    /** when this is true, ITW will not attempt any inet connections and will work only with what is in cache*/
+    private static boolean offlineForced = false;
+
+    private static Boolean onlineDetected = null;
+
 
     /** 
      * Header is not checked and so eg
@@ -379,6 +388,54 @@
         new ParserDelegator();
     }
 
+
+    public static void setOfflineForced(boolean b) {
+        offlineForced = b;
+        OutputController.getLogger().log(OutputController.Level.MESSAGE_DEBUG, "Forcing of offline set to: " + offlineForced);
+    }
+
+    public static boolean isOfflineForced() {
+        return offlineForced;
+    }
+
+    public static void setOnlineDetected(boolean online) {
+        onlineDetected = online;
+        OutputController.getLogger().log(OutputController.Level.MESSAGE_DEBUG, "Detected online set to: " + onlineDetected);
+    }
+
+    public static boolean isOnlineDetected() {
+        if (onlineDetected == null) {
+            //"file" protocol do not do online check
+            //sugest online for this case
+            return true;
+        }
+        return onlineDetected;
+    }
+
+    public static boolean isOnline() {
+        if (isOfflineForced()) {
+            return false;
+        }
+        return isOnlineDetected();
+    }
+
+    public static void detectOnline(URL location) {
+        if (onlineDetected != null) {
+            return;
+        }
+        try {
+            if (location.getProtocol().equals("file")) {
+                return;
+            }
+            //Checks the offline/online status of the system.
+            InetAddress.getByName(location.getHost());
+        } catch (UnknownHostException ue) {
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "The host of " + location.toExternalForm() + " file should be located seems down, or you are simply offline.");
+            JNLPRuntime.setOnlineDetected(false);
+            return;
+        }
+        setOnlineDetected(true);
+    }
    
     /**
      * see <a href="https://en.wikipedia.org/wiki/Double-checked_locking#Usage_in_Java">Double-checked locking in Java</a>
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/services/XBasicService.java
--- a/netx/net/sourceforge/jnlp/services/XBasicService.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/services/XBasicService.java	Tue Aug 05 17:28:31 2014 +0200
@@ -95,18 +95,8 @@
     public boolean isOffline() {
 
         URL url = findFirstURLFromJNLPFile();
-        URLConnection conn = null;
-        try {
-            conn = url.openConnection();
-            conn.getInputStream().close();
-            return false;
-        } catch (IOException exception) {
-            return true;
-        } finally {
-            if (conn != null && conn instanceof HttpURLConnection) {
-                ((HttpURLConnection) conn).disconnect();
-            }
-        }
+        JNLPRuntime.detectOnline(url);
+        return !JNLPRuntime.isOnline();
     }
 
     /**
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/util/XDesktopEntry.java
--- a/netx/net/sourceforge/jnlp/util/XDesktopEntry.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/util/XDesktopEntry.java	Tue Aug 05 17:28:31 2014 +0200
@@ -102,8 +102,8 @@
             fileContents += "Vendor=" + sanitize(file.getInformation().getVendor()) + "\n";
         }
 
-        //Shortcut executes the jnlp from cache and system preferred java..
-        fileContents += "Exec=" + "javaws" + " \"" + cacheFile.getAbsolutePath() + "\"\n";
+        //Shortcut executes the jnlp as it was with system preferred java. It should work fine offline
+        fileContents += "Exec=" + "javaws" + " \"" + file.getSourceLocation() + "\"\n";
 
         return new StringReader(fileContents);
 


More information about the distro-pkg-dev mailing list
diff -r c6e06c9b4bfb -r 178f758bd94d netx/net/sourceforge/jnlp/Launcher.java
--- a/netx/net/sourceforge/jnlp/Launcher.java	Tue Aug 05 11:13:20 2014 -0400
+++ b/netx/net/sourceforge/jnlp/Launcher.java	Tue Aug 05 17:28:31 2014 +0200
@@ -224,19 +224,21 @@
 
         JNLPRuntime.markNetxRunning();
 
-        //First checks whether offline-allowed tag is specified inside the jnlp
-        //file.
-        if (!file.getInformation().isOfflineAllowed()) {
-            try {
-                //Checks the offline/online status of the system.
-                //If system is offline do not launch.
-                InetAddress.getByName(file.getSourceLocation().getHost());
-
-            } catch (UnknownHostException ue) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "File cannot be launched because offline-allowed tag not specified and system currently offline.");
-                return null;
-            } catch (Exception e) {
-                OutputController.getLogger().log(e);
+        if (!JNLPRuntime.isOfflineForced()) {
+            //Xoffline NOT specified
+            //First checks whether offline-allowed tag is specified inside the jnlp file.
+            if (!file.getInformation().isOfflineAllowed() && !JNLPRuntime.isOnlineDetected()) {
+                {
+                    OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Remote systems unreachable, and client application is not able to run offline. Exiting.");
+                    return null;
+                }
+            }
+        } else {
+            //Xoffline IS specified
+            if (!file.getInformation().isOfflineAllowed() && !JNLPRuntime.isOnlineDetected()) {
+                {
+                    OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Remote systems unreachable, and client application is not able to run offline. However, you specified -Xoffline argument. Attmpting to run.");
+                }
             }
         }
 